% This package contains the sample code for implementation of the work on depth recovery [1][2]. Note that the sample code is run 
% with the block size of 60x60.  Too large block sizes may cause the "out of memory" problem. So, the partition-and-processing 
% strategy is needed to process large depth maps. Please contact {yexch,yjy}@tju.edu.cn for problems in using this code. See  data 
% and results at the project website at http://cs.tju.edu.cn/faculty/likun/projects/depth_recovery/index.htm
% 
% 
% [1]. Jingyu Yang, Xinchen Ye, Kun Li, Chunping Hou, Yao Wang, "Color-guided depth recovery from RGB-D data using an adaptive 
% autoregressive model", IEEE Transactions on Image Processing, vol. 23, no. 8, pp. 3443-3458, 2014.
% [2]. Jingyu Yang, Xinchen Ye, Kun Li, and Chunping Hou, "Depth recovery using an adaptive color-guided auto-regressive model", 
% European Conference on Computer Vision (ECCV), October 7-13, 2012, Firenze, Italy.  

function NLM = AR_construct(colormap_temp,guidedmap_temp, block_size,halfwindow_size,halfsearch_size)

%non-local term
w=halfwindow_size;%half size of the search window
f=halfsearch_size;%block size to diff

sigma1=4; %depth weighted
sigma_c=0.35;
sigma2=sigma_c*1.732*11; %color weighted
% sigma3=3; %gausian weighted 
sigma4=0.1;%bi weighted
% [Dx,Dy]= gradient(guidedmap_temp);
% D = sqrt(Dx.^2+Dy.^2);
% norm_D = (D-min(D(:)))/(max(D(:))-min(D(:)));
% sigma1 = 3*exp(-log(6)*norm_D);
NLM=zeros(block_size^2); 
image = padarray(colormap_temp, [f,f], 'symmetric'); 
% [X,Y] = meshgrid(-f:f,-f:f);
%G_kernel = exp(-(X.^2+Y.^2)/(2*sigma3^2));%gaussian kernal

       for t=1:block_size
            for s=1:block_size
                
                 NLM_temp=zeros(2*w+1);
                 H=zeros(block_size+2*w);
                 guided_kernel=zeros(2*w+1);
                 sMin = max(s-w,1);
                 sMax = min(s+w,block_size);
                 tMin = max(t-w,1);
                 tMax = min(t+w,block_size);
                 sigma_P=image(s:s+2*f,t:t+2*f,:); 
                 guided_kernel((sMin:sMax)-s+w+1,(tMin:tMax)-t+w+1)=...
                 exp(-(abs(guidedmap_temp(sMin:sMax,tMin:tMax)-guidedmap_temp(s,t))).^2/(2*sigma1^2));
                 dR = abs(sigma_P(:,:,1)-image(s+f,t+f,1));
                 dG = abs(sigma_P(:,:,2)-image(s+f,t+f,2));
                 dB = abs(sigma_P(:,:,3)-image(s+f,t+f,3));
                 color_kernel = exp(-(dR.^2+dG.^2+dB.^2)/(2*3*sigma4^2));
                 for k=sMin+f:sMax+f
                    for l=tMin+f:tMax+f
                        if ~(k==(s+f) && l==(t+f))
                    
                          sigma_Q=image(k-f:k+f,l-f:l+f,:);
                          diff=sum((sigma_P-sigma_Q),3);
                          kpq=exp(-sum(sum (color_kernel.*(diff.^2)))/(2*3*sigma2^2));                             
                          NLM_temp(k-f-s+w+1,l-f-t+w+1)=-kpq;
                          
                        end
                     end
                 end
                  NLM_temp=NLM_temp.*guided_kernel;
                  NLM_temp=-NLM_temp/sum(NLM_temp(:));
                  NLM_temp(w+1,w+1)=1;
                  H(s:s+2*w,t:t+2*w)=NLM_temp;
                  mask=H(w+1:end-w,w+1:end-w);
                  NLM((t-1)*block_size+s,:)=mask(:);
             end
       end
end







